# MongoDB

1. [Run MongoDB & Mongo Shell](#1-run-mongodb-mongo-shell)
2. [Import Example DataSet](#2-import-example-dataset)
3. [MongoDB Shell Help Commands](#3-mongodb-shell-help-commands)
4. [Using Mongo Shell](#4-using-mongo-shell)
5. [Data Aggregation](#5-data-aggregation)
6. [Indexes](#6-indexes)

## 1- Run MongoDB & Mongo Shell

1- Open a command prompt window and change directory to \mongodb\bin folder. Run mongod.exe.

```shell
cd \mongodb\bin
mongod
```

If your  data directory is different from \data\db then use "--dbpath" parameter to specify mongo db data folder

```shell
mongod --dbpath d:\db\MongoDB-data
```

Your mongodb server should be started on port 27017.

2- Open another command prompt window and change directory to \mongodb\bin and run mongo.exe

## 2- Import Example DataSet

1- copy file into the mongodb\bin directory.
2- open a new command prompt window and change directory to "mongodb\bin"  
3- type following command to insert sample dataset

```shell
mongoimport --db test --collection restaurants --drop --file dataset.json
```

this command will create a database named test and import data into restaurants collection.

## 3- MongoDB Shell Help Commands

1- Switch to mongo shell window
2- Type "help" and press enter button. You should see all commands and their description.
3- Type "show dbs" and press enter
4- To change current database to "test" type "use test"
5- To display collections  in current database type "show collections". You should see "restaurants" as collection name
6- Type "db.collections.help()" and inspect result. You should see all commands related to collections

## 4- Using Mongo Shell

### Insert a document

Insert a document into a collection named restaurants

```javascript
db.restaurants.insert(
   {
      "address" : {
         "street" : "2 Avenue",
         "zipcode" : "10075",
         "building" : "1480",
         "coord" : [ -73.9557413, 40.7720266 ],
      },
      "borough" : "Manhattan",
      "cuisine" : "Italian",
      "grades" : [
         {
            "date" : ISODate("2014-10-01T00:00:00Z"),
            "grade" : "A",
            "score" : 11
         },
         {
            "date" : ISODate("2014-01-16T00:00:00Z"),
            "grade" : "B",
            "score" : 17
         }
      ],
      "name" : "Vella",
      "restaurant_id" : "41704620"
   }
)
```

You should see number of rows inserted as result:
```javascript
WriteResult({ "nInserted" : 1 })
```

### Find or query data

1- Query for All Documents in a Collection:

To return all documents in a collection, call the find() method without a criteria document.

```javascript
db.restaurants.find()
```

2- Query by a Top Level Field

```javascript
db.restaurants.find( { "borough": "Manhattan" } )
```

3- Query by a Field in an Embedded Document

```javascript
db.restaurants.find( { "address.zipcode": "10075" } )
```

4- Query by a Field in an Array

```javascript
db.restaurants.find( { "grades.grade": "B" } )
```

5- Specify Conditions with Operators

```javascript
db.restaurants.find( { "grades.score": { $gt: 30 } } )
db.restaurants.find( { "grades.score": { $lt: 10 } } )
```

6- Combine Conditions

#### Logical AND

You can specify a logical conjunction (AND) for a list of query conditions by separating the conditions with a comma in the conditions document.
db.restaurants.find( { "cuisine": "Italian", "address.zipcode": "10075" } )

#### Logical OR

You can specify a logical disjunction (OR) for a list of query conditions by using the $or query operator.

```javascript
db.restaurants.find(
   { $or: [ { "cuisine": "Italian" }, { "address.zipcode": "10075" } ] }
)
```

7- Sort Query Results

```javascript
db.restaurants.find().sort( { "borough": 1, "address.zipcode": 1 } )
```

### Update data

1- Update Specific Fields

```javascript
db.restaurants.update(
    { "name" : "Juni" },
    {
      $set: { "cuisine": "American (New)" },
      $currentDate: { "lastModified": true }
    }
)
```

2- Update an Embedded Field

```javascript
db.restaurants.update(
  { "restaurant_id" : "41156888" },
  { $set: { "address.street": "istanbul" } }
)
```

3- Update Multiple Documents

```javascript
db.restaurants.update(
  { "address.zipcode": "10016", cuisine: "Other" },
  {
    $set: { cuisine: "Category To Be Determined" },
    $currentDate: { "lastModified": true }
  },
  { multi: true}
)
```

### Replace a Document

To replace the entire document except for the _id field, pass an entirely new document as the second argument to the update() method.

```javascript
db.restaurants.update(
   { "restaurant_id" : "41704620" },
   {
     "name" : "Vella 2",
     "address" : {
              "coord" : [ -73.9557413, 40.7720266 ],
              "building" : "1480",
              "street" : "2 Avenue",
              "zipcode" : "10075"
     }
   }
)
```

### Remove Data

1- Remove All Documents That Match a Condition

```javascript
db.restaurants.remove({"restaurant_id" : "41156888"} )
```

2- Use the justOne Option

By default, the remove() method removes all documents that match the remove condition. Use the justOne option to limit the remove operation to only one of the matching documents.

```javascript
db.restaurants.remove( { "borough": "Queens" }, { justOne: true }
```

##	5-Data Aggregation

1- Group Documents by a Field and Calculate Count

The following example groups the documents in the restaurants collection by the borough field and uses the $sum accumulator to count the documents for each group.

```javascript
db.restaurants.aggregate(
   [
     { $group: { "_id": "$borough", "count": { $sum: 1 } } }
   ]
);
```

2- Filter and Group Documents

```javascript
db.restaurants.aggregate(
   [
     { $match: { "borough": "Queens", "cuisine": "Brazilian" } },
     { $group: { "_id": "$address.zipcode" , "count": { $sum: 1 } } }
   ]
);
```

```javascript
db.restaurants.aggregate( 
[   
 { $group: { _id: "$cuisine", total: { $sum: 1 } } },   
 { $match: { total: { $gte: 700 } } }
 ] )
```

## 6-Indexes

### Create a Single-Field Index

Run following query and check query execution plan. You should see that there is no index usage. 

```javascript
db.restaurants.find({cuisine:"Italian"}).explain()
```

Create an ascending index on the "cuisine" field of the restaurants collection.
db.restaurants.createIndex( { "cuisine": 1 } )
Run query again. This time you should see that database uses index 

```json
"winningPlan" : {
        "stage" : "FETCH",
        "inputStage" : {
                "stage" : "IXSCAN",
                "keyPattern" : {
                        "cuisine" : 1
                },
                "indexName" : "cuisine_1",
                "isMultiKey" : false,
                "direction" : "forward",
                "indexBounds" : {
                        "cuisine" : [
                                "[\"Italian\", \"Italian\"]"
                        ]
                }
        }
}
```

### Create a compound index

Create a compound index on the "cuisine" field and the "address.zipcode" field. The index orders its entries first by ascending "cuisine" values, and then, within each "cuisine", by descending"address.zipcode" values.

```javascript
db.restaurants.createIndex( { "cuisine": 1, "address.zipcode": -1 } )
```

Run following command and check index usage:

```javascript
db.restaurants.find({cuisine:"Italian","address.zipcode":"11209"}).explain()
```

### Create multiple index and test usage of index

```javascript
db.restaurants.createIndex( { "grades.score": 1 } )
db.restaurants.find({"cuisine": "Italian","grades.score" : {$gt:10} }).explain()
```

### Geospatial Indexes (Optional)

1- Unzip dump.zip file into the mongodb\bin folder.
2- Open a new command prompt window and switch to \mongodb\bin folder. Type 

```shell
mongorestore dump
```

If import is completed successfully delete dump.zip and dump folder from bin folder.
Switch to mongodb shell window and type "show dbs". You should see newly added "geo" database. Type "use geo" to switch database

3- Type "show collections". List content of the collections: "db.states.find()  db.airports.find()"
4- Find all the airports in California. For this you need to get the California location (Polygon) and use the command $geoWithin in the query. From the shell it will look like :

```javascript
var cal = db.states.findOne(  {code : "CA"}  );
db.airports.find( 
  { 
    loc : { $geoWithin : { $geometry : cal.loc } } 
  },
  { name : 1 , type : 1, code : 1, _id: 0 } 
);
```

So the query is using the "California MultiPolygon" and looks in the airports collection to find all the airports that are in these polygons. This looks like the following image on a map:
![Map](images/map-1.png)

we are querying these documents with no index. You can run a query with the explain() to see what's going on. The $geoWithin operator does not need index but your queries will be more efficient with one so let's create the index:

```javascript
db.airports.ensureIndex( { "loc" : "2dsphere" } );
```

Run the explain and you will se the difference.

5- Suppose that you want to know what are all the adjacent states to California, for this we just need to search for all the states that have coordinates that "intersects" with California. This is done with the following query:

```javascript
var cal = db.states.findOne(  {code : "CA"}  );
db.states.find(
  { 
    loc : { $geoIntersects : { $geometry : cal.loc  }  } ,
    code : { $ne : "CA"  }  
  }, 
  { name : 1, code : 1 , _id : 0 } 
);
```
![Map](images/map-2.png)

Same as before $geoIntersect operator does not need an index to work, but it will be more efficient with the following index:

```javascript
db.states.ensureIndex( { loc : "2dsphere" } );
```

6- Last feature is related to query with proximity criteria. Let's find all the international airports that are located at less than 20km from the reservoir in NYC Central Park. For this you will be using the $near operator. So this query returns 2 airports, the closest being La Guardia, since the $near operator sorts the results by distance. Also it is important to raise here that the $near operator requires an index.

```javascript
db.airports.ensureIndex( { loc : "2dsphere" } );
db.airports.find(
  {
    loc : {
      $near : {
        $geometry : { 
          type : "Point" , 
          coordinates : [-73.965355,40.782865]  
        }, 
        $maxDistance : 20000
      }
    }, 
    type : "International"
  },
  {
    name : 1,
    code : 1,
    _id : 0
  }
);
```
